package com.sh.share.service;

import com.sh.share.entity.ShareType;
import com.sh.share.req.HistoryAddReq;
import com.sh.share.req.ShareCreateReq;
import com.sh.share.req.ShareUpdateReq;
import com.sh.share.resp.ShareResp;

import java.util.List;

/**
 * @author xiaozun
 * @since 2024/05/08/11:06
 */
public interface ShareService {

    /**
     * 创建分享内容
     *
     * @param shareCreateReq 分享创建请求对象
     */
    void createShare(ShareCreateReq shareCreateReq);

    /**
     * 删除分享内容
     *
     * @param historyAddReq 历史添加对象
     */
    void deleteShare(HistoryAddReq historyAddReq);

    /**
     * 逻辑删除过期分享
     */
    void overDue();

    /**
     * 续期
     *
     * @param shareId 分享内容id
     * @param time    增加的时间
     */
    void addDue(Long shareId, Integer time);

    /**
     * 获取所有share记录
     *
     * @return List<ShareResp>
     */
    List<ShareResp> getSharesList();

    /**
     * 获取单条share记录
     *
     * @param id 分享内容id
     * @return ShareResp
     */
    ShareResp getShareById(Long id);

    /**
     * 获取分享链接
     *
     * @param id 分享内容id
     * @return String
     */
    String getShareLink(Long id);

    /**
     * 获取密码
     *
     * @param settingId 分享记录id
     * @return String
     */
    String getPassWordBySettingId(Long settingId);

}
