package com.sh.share.service.impl;

import com.sh.share.entity.ShareType;
import com.sh.share.entity.ShareTypeExample;
import com.sh.share.entity.ShareTypePermission;
import com.sh.share.entity.ShareTypePermissionExample;
import com.sh.share.mapper.ShareTypeMapper;
import com.sh.share.mapper.ShareTypePermissionMapper;
import com.sh.share.req.AddSharePermissionReq;
import com.sh.share.req.AddShareTypeReq;
import com.sh.share.service.ShareInfoService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author xiaozun
 * @since 2024/05/08/13:17
 */
@Service
public class ShareInfoServiceImpl implements ShareInfoService {
    @Resource
    private ShareTypePermissionMapper typePermissionMapper;

    @Resource
    private ShareTypeMapper typeMapper;


    @Override
    public void fakeDelType(String typeCode) {
        ShareTypeExample typeExample = new ShareTypeExample();
        ShareType shareType = new ShareType();
        ShareTypeExample.Criteria criteria = typeExample.createCriteria();
        shareType.setDelFlag("-1");
        criteria.andCodeEqualTo(typeCode);
        typeMapper.updateByExampleSelective(shareType, typeExample);
    }

    @Override
    public void fakeDelPermission(String permissionCode) {
        ShareTypePermissionExample permissionExample = new ShareTypePermissionExample();
        ShareTypePermissionExample.Criteria criteria = permissionExample.createCriteria();
        criteria.andPermissionEnEqualTo(permissionCode);
        ShareTypePermission typePermission = new ShareTypePermission();
        typePermission.setDelFlag("-1");
        typePermissionMapper.updateByExampleSelective(typePermission, permissionExample);
    }

    @Override
    public List<ShareType> getAllShareType() {
        List<ShareType> typeList = typeMapper.selectByExample(null);
        return typeList;
    }

    @Override
    public ShareType getShareType(String typeCode) {
        ShareTypeExample typeExample = new ShareTypeExample();
        ShareTypeExample.Criteria criteria = typeExample.createCriteria();
        criteria.andCodeEqualTo(typeCode);
        List<ShareType> typeList = typeMapper.selectByExample(typeExample);
        ShareType shareType = typeList.get(0);
        return shareType;
    }

    @Override
    public Long addBitPermission(String typeCode) {
        List<Long> bitPermissions = new ArrayList<>();
        ShareTypePermissionExample shareTypePermissionExample = new ShareTypePermissionExample();
        ShareTypePermissionExample.Criteria criteria = shareTypePermissionExample.createCriteria();
        criteria.andShareTypeCodeEqualTo(typeCode);
        List<ShareTypePermission> typePermissions = typePermissionMapper.selectByExample(shareTypePermissionExample);
        if (typePermissions.isEmpty()){
            return 1L;
        }
        for (ShareTypePermission typePermission : typePermissions) {
            bitPermissions.add(typePermission.getBitPermission());
        }
        return bitPermissions.get(bitPermissions.size() - 1) << 1;
    }

    @Override
    public List<ShareTypePermission> getAllShareTypePermissionByTypeCode(String typeCode) {
        ShareTypePermissionExample permissionExample = new ShareTypePermissionExample();
        ShareTypePermissionExample.Criteria criteria = permissionExample.createCriteria();
        criteria.andShareTypeCodeEqualTo(typeCode);
        List<ShareTypePermission> typePermissions = typePermissionMapper.selectByExample(permissionExample);
        return typePermissions;
    }

    @Override
    public void addShareType(AddShareTypeReq addShareTypeReq) {
        ShareType shareType = new ShareType();
        shareType.setName(addShareTypeReq.getName());
        shareType.setCode(addShareTypeReq.getCode());
        shareType.setDescription(addShareTypeReq.getDescription());
        shareType.setDelFlag(addShareTypeReq.getDelFlag());
        shareType.setCreateTime(addShareTypeReq.getCreateTime());
        shareType.setCreateBy(addShareTypeReq.getCreateBy());
        shareType.setUpdateTime(addShareTypeReq.getUpdateTime());
        shareType.setUpdateBy(addShareTypeReq.getUpdateBy());
        shareType.setDeptId(addShareTypeReq.getDeptId());
        shareType.setGroupIndex(addShareTypeReq.getGroupIndex());
        shareType.setGroupPi(addShareTypeReq.getGroupPi());
        shareType.setTenantId(addShareTypeReq.getTenantId());
        typeMapper.insert(shareType);
    }

    @Override
    public void addPermission(AddSharePermissionReq addSharePermissionReq) {
        ShareTypePermission typePermission = new ShareTypePermission();
        typePermission.setShareTypeId(addSharePermissionReq.getShareTypeId());
        typePermission.setShareTypeCode(addSharePermissionReq.getShareTypeCode());
        typePermission.setBitPermission(addSharePermissionReq.getBitPermission());
        typePermission.setPermissionCn(addSharePermissionReq.getPermissionCn());
        typePermission.setPermissionEn(addSharePermissionReq.getPermissionEn());
        typePermission.setDescription(addSharePermissionReq.getDescription());
        typePermission.setDelFlag(addSharePermissionReq.getDelFlag());
        typePermission.setCreateTime(addSharePermissionReq.getCreateTime());
        typePermission.setCreateBy(addSharePermissionReq.getCreateBy());
        typePermission.setUpdateTime(addSharePermissionReq.getUpdateTime());
        typePermission.setUpdateBy(addSharePermissionReq.getUpdateBy());
        typePermission.setDeptId(addSharePermissionReq.getDeptId());
        typePermission.setGroupIndex(addSharePermissionReq.getGroupIndex());
        typePermission.setGroupPi(addSharePermissionReq.getGroupPi());
        typePermission.setTenantId(addSharePermissionReq.getTenantId());
        typePermissionMapper.insert(typePermission);
    }

}
