package com.sh.share.service.impl;

import com.sh.share.entity.*;
import com.sh.share.mapper.*;
import com.sh.share.req.HistoryAddReq;
import com.sh.share.req.ShareCreateReq;
import com.sh.share.req.ShareUpdateReq;
import com.sh.share.resp.ShareResp;
import com.sh.share.service.ShareInfoService;
import com.sh.share.service.ShareService;
import com.sh.share.utils.MyUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xiaozun
 * @since 2024/05/08/11:06
 */
@Service
public class ShareServiceImpl implements ShareService {
    @Resource
    private ShareMapper shareMapper;

    @Resource
    private ShareSettingsMapper settingsMapper;

    @Resource
    private ShareReceiverTypeMapper receiverTypeMapper;

    @Resource
    private ShareTypeMapper typeMapper;

    @Resource
    private ShareHistoryMapper historyMapper;

    @Resource
    private ShareInfoService infoService;


    @Override
    public void createShare(ShareCreateReq shareCreateReq) {
        Share share = new Share();
        share.setName(shareCreateReq.getName());
        share.setContentTypeCode(shareCreateReq.getContentTypeCode());
        Integer id = getTypeIdByCode(share.getContentTypeCode());
        share.setShareResourceId(id.toString());
        share.setShareResourceImage(shareCreateReq.getShareResourceImage());
        share.setShareResourceStatus(shareCreateReq.getShareResourceStatus());
        share.setSummary(shareCreateReq.getSummary());
        share.setDescription(shareCreateReq.getDescription());
        share.setVersion(shareCreateReq.getVersion());
        share.setParentVersionId(shareCreateReq.getParentVersionId());
        share.setDelFlag(shareCreateReq.getDelFlag());
        share.setCreateTime(shareCreateReq.getCreateTime());
        share.setCreateBy(shareCreateReq.getCreateBy());
        share.setUpdateTime(shareCreateReq.getUpdateTime());
        share.setUpdateBy(shareCreateReq.getUpdateBy());
        share.setDeptId(shareCreateReq.getDeptId());
        share.setGroupIndex(shareCreateReq.getGroupIndex());
        share.setGroupPi(shareCreateReq.getGroupPi());
        share.setTenantId(shareCreateReq.getTenantId());
        shareMapper.insert(share);

        ShareSettings settings = new ShareSettings();
        settings.setShareBy(shareCreateReq.getShareBy());
        settings.setShareName(shareCreateReq.getShareName());
        settings.setShareId(share.getId());
        settings.setReceiverTypeCode(shareCreateReq.getReceiverTypeCode());
        settings.setShareTime(shareCreateReq.getShareTime());
        settings.setShareExpireTime(shareCreateReq.getShareExpireTime());
        settings.setSharePermission(shareCreateReq.getSharePermission());
        settings.setEncryptedShare(shareCreateReq.getEncryptedShare());
        settings.setEncryptionPassword(shareCreateReq.getEncryptionPassword());
        settings.setAccessLink(shareCreateReq.getAccessLink());
        settings.setCreateTime(shareCreateReq.getCreateTime());
        settings.setCreateBy(shareCreateReq.getCreateBy());
        settings.setUpdateTime(shareCreateReq.getUpdateTime());
        settings.setTenantId(shareCreateReq.getTenantId());
        settings.setDeptId(shareCreateReq.getDeptId());
        settings.setGroupIndex(shareCreateReq.getGroupIndex());
        settings.setGroupPi(shareCreateReq.getGroupPi());
        List<String> sharedToIds = shareCreateReq.getSharedToIds();
        if (sharedToIds == null){
            settings.setSharedToId(null);
            settingsMapper.insert(settings);
        }else {
            for (String sharedToId : shareCreateReq.getSharedToIds()) {
                settings.setSharedToId(sharedToId);
                settingsMapper.insert(settings);
            }
        }
    }

    private Integer getTypeIdByCode(String code){
        ShareTypeExample typeExample = new ShareTypeExample();
        ShareTypeExample.Criteria criteria = typeExample.createCriteria();
        criteria.andCodeEqualTo(code);
        List<ShareType> shareTypes = typeMapper.selectByExample(typeExample);
        Integer id = shareTypes.get(0).getId();
        return id;
    }


    @Override
    public void deleteShare(HistoryAddReq historyAddReq) {
        ShareSettingsExample settingsExample = new ShareSettingsExample();
        Long shareId = historyAddReq.getId();
        settingsExample.createCriteria().andShareIdEqualTo(shareId);
        List<ShareSettings> shareSettings = settingsMapper.selectByExample(settingsExample);
        for (ShareSettings settings : shareSettings) {
            Share share = shareMapper.selectByPrimaryKey(historyAddReq.getId());
            historyAddReq.setContent(share.getSummary());
            historyAddReq.setCreateTime(settings.getCreateTime());
            historyAddReq.setCreateBy(settings.getCreateBy());
            historyAddReq.setUpdateTime(settings.getUpdateTime());
            historyAddReq.setUpdateBy(settings.getUpdateBy());
            historyAddReq.setDeptId(settings.getDeptId());
            historyAddReq.setGroupIndex(settings.getGroupIndex());
            historyAddReq.setGroupPi(settings.getGroupPi());
            historyAddReq.setTenantId(settings.getTenantId());

            ShareHistory shareHistory = new ShareHistory();
            shareHistory.setShareId(shareId);
            shareHistory.setOperatorUsername(historyAddReq.getOperatorUsername());
            shareHistory.setOperatorTime(historyAddReq.getOperatorTime());
            shareHistory.setContent(historyAddReq.getContent());
            shareHistory.setDelFlag(historyAddReq.getDelFlag());
            shareHistory.setCreateTime(historyAddReq.getCreateTime());
            shareHistory.setCreateBy(historyAddReq.getCreateBy());
            shareHistory.setUpdateTime(historyAddReq.getUpdateTime());
            shareHistory.setUpdateBy(historyAddReq.getUpdateBy());
            shareHistory.setDeptId(historyAddReq.getDeptId());
            shareHistory.setGroupIndex(historyAddReq.getGroupIndex());
            shareHistory.setGroupPi(historyAddReq.getGroupPi());
            shareHistory.setTenantId(historyAddReq.getTenantId());

            settingsMapper.deleteByExample(settingsExample);
            historyMapper.insert(shareHistory);
        }

        shareMapper.deleteByPrimaryKey(shareId);
    }


    @Override
    public void overDue() {
        ShareSettingsExample settingsExample = new ShareSettingsExample();
        ShareSettingsExample.Criteria settingCriteria = settingsExample.createCriteria();
        settingCriteria.andShareExpireTimeLessThan(new Date());

        List<ShareSettings> shareSettings = settingsMapper.selectByExample(settingsExample);

        ShareExample shareExample = new ShareExample();
        ShareExample.Criteria exampleCriteria = shareExample.createCriteria();

        for (ShareSettings shareSetting : shareSettings) {
            Share share = new Share();
            Long shareId = shareSetting.getShareId();
            share.setDelFlag("-1");
            exampleCriteria.andIdEqualTo(shareId);
            shareMapper.updateByExampleSelective(share, shareExample);
        }
    }

    @Override
    public void addDue(Long shareId, Integer time) {
        Share share = shareMapper.selectByPrimaryKey(shareId);

        Date updateTime = share.getUpdateTime();
        Date date = MyUtils.addDate(updateTime, time);

        ShareSettingsExample settingsExample = new ShareSettingsExample();
        ShareSettingsExample.Criteria settingCriteria = settingsExample.createCriteria();
        settingCriteria.andShareIdEqualTo(shareId);
        List<ShareSettings> settings = settingsMapper.selectByExample(settingsExample);
        for (ShareSettings setting : settings) {
            setting.setShareExpireTime(date);
            setting.setUpdateTime(updateTime);
            settingsMapper.updateByExampleSelective(setting,settingsExample);
        }

        ShareExample shareExample = new ShareExample();
        ShareExample.Criteria exampleCriteria = shareExample.createCriteria();
        share.setDelFlag("0");
        exampleCriteria.andIdEqualTo(shareId);
        shareMapper.updateByExampleSelective(share, shareExample);
    }

    @Override
    public List<ShareResp> getSharesList() {
        List<Share> shares = shareMapper.selectByExample(null);
        List<ShareResp> shareResps = new ArrayList<>();
        for (Share share : shares) {
            ShareSettingsExample settingsExample = new ShareSettingsExample();
            ShareSettingsExample.Criteria criteria = settingsExample.createCriteria();
            criteria.andShareIdEqualTo(share.getId());
            List<ShareSettings> settings = settingsMapper.selectByExample(settingsExample);
            for (ShareSettings setting : settings) {
                ShareResp resp = new ShareResp();
                resp.setShare(share);
                resp.setShareSettings(setting);
                shareResps.add(resp);
            }
        }
        return shareResps;
    }

    @Override
    public ShareResp getShareById(Long id) {
        ShareResp shareResp = new ShareResp();
        ShareSettings settings = settingsMapper.selectByPrimaryKey(id);
        Long shareId = settings.getShareId();
        Share share = shareMapper.selectByPrimaryKey(shareId);
        shareResp.setShare(share);
        shareResp.setShareSettings(settings);
        return shareResp;
    }

    @Override
    public String getShareLink(Long id) {
        ShareSettings settings = settingsMapper.selectByPrimaryKey(id);
        return settings.getAccessLink();
    }

    @Override
    public String getPassWordBySettingId(Long settingId) {
        ShareSettings shareSetting = settingsMapper.selectByPrimaryKey(settingId);
        return shareSetting.getEncryptionPassword();
    }
}
